@@ -89,6 +89,30 @@ Usage: foo <first> <second-and-third> <second-and-third>
       )
    end)
 
+   it("creates correct usage message for arguments with choices", function()
+      local parser = Parser "foo"
+         :add_help(false)
+      parser:argument "move"
+         :choices {"rock", "paper", "scissors"}
+
+      assert.equal(
+         [=[Usage: foo {rock,paper,scissors}]=],
+         parser:get_usage()
+      )
+   end)
+
+   it("creates correct usage message for options with argument choices", function()
+      local parser = Parser "foo"
+         :add_help(false)
+      parser:option "--format"
+         :choices {"short", "medium", "full"}
+
+      assert.equal(
+         [=[Usage: foo [--format {short,medium,full}]]=],
+         parser:get_usage()
+      )
+   end)
+
    it("creates correct usage message for commands", function()
       local parser = Parser "foo"
          :add_help(false)
@@ -320,7 +344,7 @@ Usage: foo ([--opt1 <opt1>] | [--opt3 <opt3>] | [--opt5 <opt5>])
          :args "*"
 
       assert.equals([=[
-Usage: foo [--set-important-property <set_important_property>] [-h]
+Usage: foo [-h] [--set-important-property <set_important_property>]
   <long_argument_name> <very_long_words> [--include [<include>] ...]]=], parser:get_usage())
    end)
 
@@ -336,9 +360,9 @@ Usage: foo [--set-important-property <set_important_property>] [-h]
          :args "*"
 
       assert.equals([=[
-Usage: foo
+Usage: foo [-h]
        [--set-important-property <set_important_property>]
-       [-h] <long_argument_name> <very_long_words>
+       <long_argument_name> <very_long_words>
        [--include [<include>] ...]]=], parser:get_usage())
    end)
 end)
